/********************************************************
Workspace: IAR+SPLib
Chip type: STM8S003K3 (16MHz HSI)
Project  : softstart for chainsaw
Version  : 2
Date     : Nov 2017
*********************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include <stdbool.h>
#include "bitset.c"
#include "defines.c"
#include "periph_init.c"
#include "I_calculations.c"
#include "scale_work.c"

volatile u16 tim0,tim1;
volatile bool iwdg_t2, iwdg_t4;

bool softstart=0;
volatile bool strobe=0;

u16 scale_val;

void devcon();


extern "C"
{
 INTERRUPT_HANDLER(TIM2_CAPCOM, 14)
 {
  if(!TIM2->CCR2H) goto skip_t2c; 
  
  TIM1->EGR = TIM1_EGR_UG;
  TIM2->EGR = TIM2_EGR_UG;
  TIM1->CR1 |= TIM1_CR1_CEN;   
  
  u16 tmp; 
  tmp  = TIM2->CCR2H<<8;
  tmp |= TIM2->CCR2L;
  tmp >>= 1;   
  
  TIM1->ARRH = tmp>>8;
  TIM1->ARRL = (u8)tmp;

  static u8 scr_angle;  
  if(softstart)
  {
   if(scr_angle<0xff-1)scr_angle+=2;
  }
  else scr_angle = idle_val;
  
  if(scr_angle < 0xff) tmp = tmp-((u32)tmp*scr_angle>>8);
  else tmp = 0;
  TIM1->CCR2H =TIM1->CCR1H = tmp>>8;
  TIM1->CCR2L =TIM1->CCR1L = (u8)tmp;  

  strobe = 1;
  
  if(tim0)tim0--;
  if(tim1)tim1--;  
  
  PE_ODR^=0xff;
  iwdg_t2 = 1;
  
  skip_t2c:
  TIM2->SR1 = 0;
 }
 
 INTERRUPT_HANDLER(TIM4_OVF, 23)   //12500Hz
 {
  static u32 adc_accm;
  static u16 adc_cnt;
  static u16 adc_avr_cnt;
  
  ADC1->CSR  = 1;
  ADC1->CR1 |= ADC1_CR1_ADON; 

  u16 tmp = *((u16*)&ADC1->DB0RH); 
  adc_ref_retval  = *((u16*)&ADC1->DB1RH);
  
  if(tmp)
  {
   adc_accm += tmp; 
   adc_cnt++;  
  } 
  adc_avr_cnt++;
    
  if(strobe)
  {
   strobe=0;
   
   if(!adc_ready)
   {
    adc_ready = 1;

    adc_accm_retval = adc_accm;
    adc_cnt_retval  = adc_cnt; 
    
    avr_cnt_retval  = adc_avr_cnt;
    adc_accm = 0;
    adc_cnt  = 0;
    adc_avr_cnt = 0;
   }
  }
  
  static u8 tcnt;
  if(!tcnt)
  {
   tcnt = 12500/100;
   PC_ODR_ODR3^=1;
  }
  else tcnt--;
  
  iwdg_t4=1;
  TIM4->SR1 = 0;
 }
}

void main()
{
 cfg.gpio();
 cfg.clk();
 cfg.adc(); 
 cfg.t1();
 cfg.t2();
 cfg.t4();
 cfg.ee();

 cfg.iwdg();


 ITC_SetSoftwarePriority(ITC_IRQ_TIM2_CAPCOM, ITC_PRIORITYLEVEL_1);
 ITC_SetSoftwarePriority(ITC_IRQ_TIM4_OVF, ITC_PRIORITYLEVEL_0);

 asm("rim");

m:
 {
  devcon();  
  display_scale(scale_val);

  if(iwdg_t2&&iwdg_t4){iwdg_t2=iwdg_t4=0;IWDG_ReloadCounter();}
 }
goto m;
}


void devcon()
{
 static bool itr=0;
 static u8 ocp_cnt=0; 
 static enum {wfl,work,ocp}dev_st = wfl;

 switch (dev_st)
 {
  case wfl:
  {
   if(!itr)
   {
    itr=1;
    softstart = 0;
    scale_val = 0;
    TIM1->BKR |= TIM1_BKR_MOE;
   } 
   
   static u16 I;
   if(get_I(&I)==1)
   {
    if(I>eep.I_Htrs)      dev_st = ocp;
    else if(I>eep.I_Ltrs) dev_st = work; 
    else
    {
     if(!tim1)
     {
      tim1 = 50;
      scale_val ^= 1;
     }
    }    
   }   

   if(dev_st!=wfl){itr=0;}
  }
  break;

  case work:
  {
   if(!itr)
   {
    itr=1;
    softstart = 1;
   } 
   
   u16 I;
   if(get_I(&I)==1)
   {
    if(I>eep.I_Htrs)      dev_st = ocp;
    else if(I<eep.I_Ltrs) dev_st = wfl; 
    
    if(!tim1)
    {
     static u16 tmp_I = eep.I_max*2/3;
    
     tim1 = 25;
     
     if(I<tmp_I) scale_val = data_2_scale(I, tmp_I);
     else
     {
      static bool scale_trg;
      if(scale_trg^=1) 
      { 
       scale_val = 0x3ff; 
      } 
      else 
      { 
       scale_val = 0x7f; 
      } 
     }
    }    
   }
   
   if(dev_st!=work){itr=0;}
  }
  break;
  
  case ocp:
  {
   if(!itr)
   {
    itr=1;
    TIM1->BKR &= ~TIM1_BKR_MOE;
    scale_val = 0x380;
    if(!tim0)ocp_cnt=0;
    tim0 = 4*50;
   }   
   
   if(!tim0 && ocp_cnt++<5)
   {
    dev_st=wfl;
    tim0 = 4*50;
   }
   
   if(dev_st!=ocp){itr=0;}
  }
  break; 
  
  default:break;
 }

}

